@@ -19,11 +19,13 @@ from .pygeoutils import (
     arcgis2geojson,
     break_lines,
     geo2polygon,
+    geodf2xarray,
     geometry_list,
     get_gtiff_attrs,
     get_transform,
     gtiff2xarray,
     json2geodf,
+    nested_polygons,
     snap2nearest,
     xarray2geodf,
     xarray_geomask,
@@ -44,11 +46,13 @@ __all__ = [
     "gtiff2xarray",
     "snap2nearest",
     "xarray2geodf",
+    "geodf2xarray",
     "json2geodf",
     "transform2tuple",
     "xarray_geomask",
     "Coordinates",
     "GeoBSpline",
+    "nested_polygons",
     "InputTypeError",
     "InputValueError",
     "InputRangeError",
