@@ -1,7 +1,6 @@
 """Some utilities for manipulating GeoSpatial data."""
 from __future__ import annotations
 
-import contextlib
 import tempfile
 import uuid
 from dataclasses import dataclass
@@ -55,8 +54,10 @@ __all__ = [
     "xarray_geomask",
     "gtiff2xarray",
     "xarray2geodf",
+    "geodf2xarray",
     "Coordinates",
     "GeoBSpline",
+    "nested_polygons",
 ]
 
 
@@ -164,6 +165,20 @@ def geo2polygon(
     return geom
 
 
+def _write_crs(ds: XD)->XD:
+    """Write geo reference info into a dataset or dataarray."""
+    ds = ds.rio.write_transform()
+    if ds.rio.grid_mapping and ds.rio.grid_mapping != "spatial_ref":
+        ds = ds.rio.write_crs(ds.rio.crs, grid_mapping_name=ds.rio.grid_mapping)
+        if "spatial_ref" in ds:
+            ds = ds.drop_vars(["spatial_ref"])
+    else:
+        ds = ds.rio.write_crs(ds.rio.crs)
+    ds = ds.rio.write_coordinate_system()
+    return ds
+
+
+
 def xarray_geomask(
     ds: XD,
     geometry: GTYPE,
@@ -208,10 +223,13 @@ def xarray_geomask(
         raise MissingCRSError
 
     geom = geo2polygon(geometry, crs, ds.rio.crs)
+    ds = _write_crs(ds)
     ds = ds.rio.clip_box(*geom.bounds)
     if isinstance(geometry, (Polygon, MultiPolygon)):
         ds = ds.rio.clip([geom], all_touched=all_touched, drop=drop, from_disk=from_disk)
 
+    if drop:
+        ds = _write_crs(ds)
     ds.rio.update_attrs(ds_attrs, inplace=True)
     if isinstance(ds, xr.Dataset):
         _ = [ds[v].rio.update_attrs(da_attrs[v], inplace=True) for v in ds]
@@ -336,7 +354,7 @@ def gtiff2xarray(
             memfile.write(resp)
             with memfile.open(driver=driver) as vrt:
                 ds = rxr.open_rasterio(vrt)
-                with contextlib.suppress(ValueError):
+                if "band" in ds.dims:
                     ds = ds.squeeze("band", drop=True)
                 ds.name = var_name[lyr]
                 dtypes[ds.name] = ds.dtype
@@ -353,28 +371,34 @@ def gtiff2xarray(
             parallel=True,
             engine="rasterio",
         )
+
+    if "band" in ds.dims:
         ds = ds.squeeze("band", drop=True)
 
-        variables = list(ds)
+    variables = list(ds)
 
-        if len(variables) == 1:
-            ds = ds[variables[0]].copy()
-            ds = ds.astype(dtypes[variables[0]])
-            ds.attrs["crs"] = attrs.crs.to_string()
-            ds.attrs["nodatavals"] = (nodata_dict[ds.name],)
-            ds = ds.rio.write_nodata(nodata_dict[ds.name])
-        else:
-            ds.attrs["crs"] = attrs.crs.to_string()
-            for v in variables:
-                ds[v] = ds[v].astype(dtypes[v])
-                ds[v].attrs["crs"] = attrs.crs.to_string()
-                ds[v].attrs["nodatavals"] = (nodata_dict[v],)
-                ds[v] = ds[v].rio.write_nodata(nodata_dict[v])
+    if len(variables) == 1:
+        ds = ds[variables[0]].copy()
+        ds = ds.astype(dtypes[variables[0]])
+        ds.attrs["crs"] = attrs.crs.to_string()
+        ds.attrs["nodatavals"] = (nodata_dict[ds.name],)
+        ds = ds.rio.write_nodata(nodata_dict[ds.name])
+    else:
+        ds.attrs["crs"] = attrs.crs.to_string()
+        for v in variables:
+            ds[v] = ds[v].astype(dtypes[v])
+            ds[v].attrs["crs"] = attrs.crs.to_string()
+            ds[v].attrs["nodatavals"] = (nodata_dict[v],)
+            ds[v] = ds[v].rio.write_nodata(nodata_dict[v])
+
+    ds = ds.rio.write_transform()
+    ds = ds.rio.write_crs(attrs.crs)
+    ds = ds.rio.write_coordinate_system()
 
     if isinstance(geometry, (Polygon, MultiPolygon)):
         if geo_crs is None:
             raise MissingCRSError
-        return xarray_geomask(ds, geometry, geo_crs, all_touched, drop, from_disk=True)
+        return xarray_geomask(ds, geometry, geo_crs, all_touched, drop)
     return ds
 
 
@@ -467,6 +491,84 @@ def xarray2geodf(
     )
 
 
+def geodf2xarray(
+    geodf: GDF,
+    resolution: float,
+    attr_col: str | None = None,
+    fill: int | float = 0,
+    projected_crs: CRSTYPE = 5070,
+) -> xr.Dataset:
+    """Rasterize a ``geopandas.GeoDataFrame`` to ``xarray.DataArray``.
+
+    Parameters
+    ----------
+    geodf : geopandas.GeoDataFrame or geopandas.GeoSeries
+        GeoDataFrame or GeoSeries to rasterize.
+    resolution : float
+        Target resolution of the output raster in the ``projected_crs`` unit. Since
+        the default ``projected_crs`` is ``EPSG:5070``, the default unit for the
+        resolution is meters.
+    attr_col : str, optional
+        Column name of the attribute to use as variable., defaults to ``None``,
+        i.e., the variable will be a boolean mask where 1 indicates the presence of
+        a geometry. Also, note that the attribute must be numeric and have one of the
+        following ``numpy`` types: ``int16``, ``int32``, ``uint8``, ``uint16``,
+        ``uint32``, ``float32``, and ``float64``.
+    fill : int or float, optional
+        Value to use for filling the missing values (mask) of the output raster,
+        defaults to ``0``.
+    projected_crs : int, str, or pyproj.CRS, optional
+        A projected CRS to use for the output raster, defaults to ``EPSG:5070``.
+
+    Returns
+    -------
+    xarray.Dataset
+        The xarray Dataset with a single variable.
+    """
+    if not pyproj.CRS(projected_crs).is_projected:
+        raise InputTypeError("projected_crs", "a projected CRS")
+
+    gdf = geodf.to_crs(projected_crs) if geodf.crs != pyproj.CRS(projected_crs) else geodf
+    west, south, east, north = gdf.total_bounds
+    width = np.ceil(abs(west - east) / resolution).astype(int)
+    height = np.ceil(abs(north - south) / resolution).astype(int)
+    affine = rio.transform.from_bounds(west, south, east, north, width, height)
+
+    if attr_col:
+        _types = ["int16", "int32", "uint8", "uint16", "uint32", "float32", "float64"]
+        valid_types = [np.dtype(t) for t in _types]
+        dtype = geodf[attr_col].dtype
+        if dtype not in valid_types:
+            raise InputTypeError("attr_col", ", ".join(_types))
+
+        ds = xr.DataArray(
+            rio.features.rasterize(
+                shapes=zip(gdf.geometry, gdf[attr_col]),
+                out_shape=(height, width),
+                transform=affine,
+                dtype=dtype,
+                fill=fill,
+            ),
+            coords={"x": np.linspace(west, east, width), "y": np.linspace(north, south, height)},
+            dims=("y", "x"),
+            name=attr_col,
+        )
+    else:
+        ds = xr.DataArray(
+            rio.features.rasterize(
+                shapes=gdf.geometry,
+                out_shape=(height, width),
+                transform=affine,
+            ),
+            coords={"x": np.linspace(west, east, width), "y": np.linspace(north, south, height)},
+            dims=("y", "x"),
+        )
+    ds = ds.rio.write_transform(affine)
+    ds = ds.rio.write_crs(projected_crs)
+    ds = ds.rio.write_coordinate_system()
+    return ds
+
+
 @dataclass
 class Coordinates:
     """Generate validated and normalized coordinates in WGS84.
@@ -859,3 +961,29 @@ def geometry_list(
         "MultiLineString",
     ]
     raise InputTypeError("geometry", ", ".join(valid_geoms))
+
+
+def nested_polygons(gdf: gpd.GeoDataFrame | gpd.GeoSeries) -> dict[int | str, list[int | str]]:
+    """Get nested polygons in a GeoDataFrame.
+
+    Parameters
+    ----------
+    gdf : geopandas.GeoDataFrame or geopandas.GeoSeries
+        A GeoDataFrame or GeoSeries with (multi)polygons.
+
+    Returns
+    -------
+    dict
+        A dictionary where keys are indices of larger ploygons and
+        values are a list of indices of smaller polygons that are
+        contained within the larger polygons.
+    """
+    if gdf.geom_type.str.contains("Polygon").sum() != len(gdf):
+        raise InputTypeError("gdf", "dataframe with (Multi)Polygons")
+
+    gdf = gdf.to_frame() if isinstance(gdf, gpd.GeoSeries) else gdf
+    q_idx, t_idx = gdf.sindex.query_bulk(gdf.geometry, predicate="within")
+    merged_idx = tlz.merge_with(
+        list, ({gdf.iloc[t].name: gdf.iloc[q].name} for q, t in zip(q_idx, t_idx) if t != q)
+    )
+    return merged_idx  # type: ignore[no-any-return]
